## $Id: browser.py 16639 2021-09-24 09:06:57Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import httplib
import hashlib
import grok
from kofacustom.nigeria.interswitch.browser import (
    module_activated,
    InterswitchPaymentRequestWebservicePageApplicant,
    InterswitchPaymentRequestWebservicePageStudent,
    InterswitchPaymentVerifyWebservicePageApplicant,
    InterswitchPaymentVerifyWebservicePageStudent,
    InterswitchPageStudent, InterswitchPageApplicant,
    )
from kofacustom.edocons.students.interfaces import ICustomStudentOnlinePayment
from kofacustom.edocons.applicants.interfaces import ICustomApplicantOnlinePayment
from kofacustom.edocons.interfaces import MessageFactory as _

PRODUCT_ID = '22133007' # must be provided by Interswitch
SITE_NAME = 'edocons.waeup.org'
PROVIDER_ACCT = '0213065415'
PROVIDER_BANK_ID = '47'
PROVIDER_ITEM_NAME = 'WAeAC'
INSTITUTION_NAME = 'EDOCONS'
CURRENCY = '566'
GATEWAY_AMT = 300.0
#MAC = 'CEF793CBBE838AA0CBB29B74D571113B4EA6586D3BA77E7CFA0B95E278364EFC4526ED7BD255A366CDDE11F1F607F0F844B09D93B16F7CFE87563B2272007AB3' # must be provided by Interswitch
MAC ='chSVbuS0CN6yg08QaX2DrlhjOpGqtxd3JojMrIrpPMArNQx4C2AcczZyGjXnhmHGzfiHe9diKGB8fTIIzifzpv20a28EYOLqdWDaB2uMyxwrnTf3ncbOx35gLIavrx1r '

POST_ACTION = 'https://webpay.interswitchng.com/collections/w/pay'
#POST_ACTION = 'https://sandbox.interswitchng.com/webpay/pay'
HOST = 'webpay.interswitchng.com'
#HOST = 'sandbox.interswitchng.com'
URL = '/collections/api/v1/gettransaction.json'
#URL = '/webpay/api/v1/gettransaction.json'

httplib.HTTPSConnection.debuglevel = 0
HTTPS = True

class CustomInterswitchPageStudent(InterswitchPageStudent):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomStudentOnlinePayment)
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    product_id = PRODUCT_ID
    mac = MAC
    gateway_amt = GATEWAY_AMT

    def update(self):
        if not module_activated(
            self.context.student.current_session, self.context):
            self.flash(_('Forbidden'), type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        # Already now it becomes an Interswitch payment. We set the net amount
        # and add the gateway amount.
        if not self.context.r_company:
            self.context.net_amt = self.context.amount_auth
            self.context.amount_auth += self.gateway_amt
            self.context.gateway_amt = self.gateway_amt
            self.context.r_company = u'interswitch'
        student = self.student
        xmldict = self.xmldict
        # Provider data
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        # Institution data
        xmldict['institution_acct'] = '00000000'
        xmldict['institution_bank_id'] = '00'
        provider_amt = 0.0
        self.pay_item_id = '0000' # must be provided by Interswitch
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['institution_item_name'] = self.context.category
        xmldict['institution_name'] = INSTITUTION_NAME
        xmldict['institution_amt'] = 100 * self.context.net_amt
        if not self.context.provider_amt:
            self.context.provider_amt = provider_amt
            self.context.amount_auth += provider_amt
        if provider_amt == 0:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        else:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.amount_auth = int(100 * self.context.amount_auth)
        hashargs = (
            self.context.p_id +
            PRODUCT_ID +
            self.pay_item_id +
            str(int(self.amount_auth)) +
            self.site_redirect_url +
            self.mac)
        self.hashvalue = hashlib.sha512(hashargs).hexdigest()
        return

class CustomInterswitchPageApplicant(InterswitchPageApplicant):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomApplicantOnlinePayment)
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    pay_item_id = 'Default_Payable_MX26002' # must be provided by Interswitch
    product_id = PRODUCT_ID
    mac = MAC
    gateway_amt = GATEWAY_AMT

    def update(self):
        if not module_activated(
            self.context.__parent__.__parent__.year, self.context):
            self.flash(_('Forbidden'), type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        # Already now it becomes an Interswitch payment. We set the net amount
        # and add the gateway amount.
        if not self.context.r_company:
            self.context.net_amt = self.context.amount_auth
            self.context.amount_auth += self.gateway_amt
            self.context.gateway_amt = self.gateway_amt
            self.context.r_company = u'interswitch'
        xmldict = {}
        provider_amt = 500.0
        xmldict['institution_acct'] = '1489560452'
        xmldict['institution_bank_id'] = '31'
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        xmldict['institution_item_name'] = self.context.category
        xmldict['institution_name'] = INSTITUTION_NAME
        midwife_amt = 200.0
        xmldict['midwife_amt'] = 100 * midwife_amt
        xmldict['institution_amt'] = 100 * (self.context.net_amt - midwife_amt)
        if not self.context.provider_amt:
            self.context.provider_amt = provider_amt
            self.context.amount_auth += provider_amt
        xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
<item_detail item_id="3" item_name="Nursing and Midwifery Committee" item_amt="%(midwife_amt)d" bank_id="8" acct_num="2004854377" />
</item_details>
</payment_item_detail>""" % xmldict
        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.amount_auth = int(100 * self.context.amount_auth)
        hashargs = (
            self.context.p_id +
            PRODUCT_ID +
            self.pay_item_id +
            str(int(self.amount_auth)) +
            self.site_redirect_url +
            self.mac)
        self.hashvalue = hashlib.sha512(hashargs).hexdigest()
        return

class CustomInterswitchPaymentRequestWebservicePageStudent(
    InterswitchPaymentRequestWebservicePageStudent):
    """Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC

class CustomInterswitchPaymentVerifyWebservicePageStudent(
    InterswitchPaymentVerifyWebservicePageStudent):
    """Payment verify view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC

class CustomInterswitchPaymentRequestWebservicePageApplicant(
    InterswitchPaymentRequestWebservicePageApplicant):
    """Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC

class CustomInterswitchPaymentVerifyWebservicePageApplicant(
    InterswitchPaymentVerifyWebservicePageApplicant):
    """Payment verify view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
    mac = MAC